/* 
 * File:   main.c
 * Author: Jelora
 *
 * Created on 18 novembre 2018, 01:52
 */

// CONFIG
#pragma config FOSC = HS        // Oscillator Selection bits (HS oscillator)
#pragma config WDTE = OFF       // Watchdog Timer Enable bit (WDT disabled)
#pragma config PWRTE = OFF      // Power-up Timer Enable bit (PWRT disabled)
#pragma config BOREN = OFF      // Brown-out Reset Enable bit (BOR disabled)
#pragma config LVP = OFF        // Low-Voltage (Single-Supply) In-Circuit Serial Programming Enable bit (RB3 is digital I/O, HV on MCLR must be used for programming)
#pragma config CPD = OFF        // Data EEPROM Memory Code Protection bit (Data EEPROM code protection off)
#pragma config WRT = OFF        // Flash Program Memory Write Enable bits (Write protection off; all program memory may be written to by EECON control)
#pragma config CP = OFF         // Flash Program Memory Code Protection bit (Code protection off)

#include <stdio.h>
#include <stdlib.h>
#include <pic16f873a.h>
#include <pic16f876a.h>

unsigned char busData = 0;
unsigned char busControl[2];
unsigned char busAdress[2];

char bufferCommande[4];

void pause();
void envoi(char octet);
void envoi_test(unsigned char valeur);
char verifCommande(char bufferCommande[], char commandeAttendue[], char longueur);
void verifReception();

int main(int argc, char** argv) {
    
    TRISA = 0b00000000;
    TRISB = 0b11111111;
    TRISC = 0b10000000;

    // Config liaison série
    //SPBRG = 25; // 9600 bits/s
    SPBRG = 12; // 19200 bits/s
    TXSTA = 0b00100100;
    RCSTA = 0b10010000;
    
    PORTA = 0b00000001;
    PORTB = 0b00000000;
    PORTC = 0b10011111;
    
    
    // Valeurs précédentes du dernier envoi
    unsigned char precedBusData = 0;
    unsigned char precedBusControl[2];
    unsigned char precedBusAdress[2];
    
    while ( 1 ){
        // Lecture d'un instanté du bus de données, d'adresse et de contrôle par les 74LS373
        PORTCbits.RC5 = 1;
        PORTCbits.RC5 = 0;
        
        // Récupération valeur du bus de données
        PORTCbits.RC0 = 0;
        busData = PORTB;
        PORTCbits.RC0 = 1;
        
        // Récupération de la 1ère partie du bus de contrôle
        PORTCbits.RC1 = 0;
        busControl[0] = PORTB;
        PORTCbits.RC1 = 1;
        
        // Récupération de la 2e partie du bus de contrôle
        PORTCbits.RC2 = 0;
        busControl[1] = PORTB;
        PORTCbits.RC2 = 1;
        
        // Récupération des 8 bits inférieurs du bus d'adressage
        PORTCbits.RC3 = 0;
        busAdress[0] = PORTB;
        PORTCbits.RC3 = 1;
        
        // Récupération des 8 bits supérieurs du bus d'adressage
        PORTCbits.RC4 = 0;
        busAdress[1] = PORTB;
        PORTCbits.RC4 = 1;
        
        // Envoi
        if ( precedBusData != busData || 
                precedBusControl[0] != busControl[0] || precedBusControl[1] != busControl[1] ||
                precedBusAdress[0] != busAdress[0] || precedBusAdress[1] != busAdress[1] ){
            
            envoi('L');
            envoi('E');
            envoi('C');
            envoi('T');
            envoi(busData);
            envoi(busControl[0]);
            envoi(busControl[1]);
            envoi(busAdress[0]);
            envoi(busAdress[1]);
            
            precedBusData = busData;
            precedBusControl[0] = busControl[0];
            precedBusControl[1] = busControl[1];
            precedBusAdress[0] = busAdress[0];
            precedBusAdress[1] = busAdress[1];
        }
        
        verifReception();
    }
    
    
    return (EXIT_SUCCESS);
}

void verifReception(){
    // Réception
    if ( RCSTAbits.OERR ){
        RCSTAbits.CREN = 0;
        RCSTAbits.CREN = 1;
    }
    if ( PIR1bits.RCIF ){
            
        bufferCommande[0] = bufferCommande[1];
        bufferCommande[1] = bufferCommande[2];
        bufferCommande[2] = bufferCommande[3];
        bufferCommande[3] = RCREG;
            
        /*if ( verifCommande(bufferCommande, "TES", 3) > 0 ){
            TXREG = 'A';
            while ( !TXSTAbits.TRMT );
        }*/
            
        // désactivation sortie sur bus de données
        if ( verifCommande(bufferCommande, "DDA", 3) > 0 ){
            PORTAbits.RA0 = 1;
        }
            
        // Ecriture et activation sortie sur bus de données
        if ( verifCommande(bufferCommande, "WED*", 4) > 0 ){
            // Port B en sortie
            TRISB = 0b00000000;
            // application de la valeur sur le 74LS373
            PORTB = bufferCommande[3];
            PORTAbits.RA1 = 1; // LE
            PORTAbits.RA1 = 0;
            // activation sortie sur bus de données
            PORTAbits.RA0 = 0; // OE
                
            // remise de Port B en entree
            PORTB = 0;
            TRISB = 0b11111111;
        }
    }
}

char verifCommande(char bufferCommande[], char commandeAttendue[], char longueur){
    char result = 1;
    for ( char i = 0 ; i < longueur ; i++ ){
        if ( bufferCommande[(4 - longueur) + i] != commandeAttendue[i] && commandeAttendue[i] != '*' ){
            result = 0;
        }
    }
    return result;
}

void envoi(char octet){
    TXREG = octet;
    while ( !TXSTAbits.TRMT );
    
    verifReception();
}

void pause(){
    int i;
    for ( i = 0 ; i < 2000 ; i++ );
}

void envoi_test(unsigned char valeur){
    TXREG = '>';
    while ( !TXSTAbits.TRMT );
    if ( (valeur & 0b10000000) > 0 ){
        TXREG = '1';
    }
    else{
        TXREG = '0';
    }
    while ( !TXSTAbits.TRMT );
    if ( (valeur & 0b01000000) > 0 ){
        TXREG = '1';
    }
    else{
        TXREG = '0';
    }
    while ( !TXSTAbits.TRMT );
    if ( (valeur & 0b00100000) > 0 ){
        TXREG = '1';
    }
    else{
        TXREG = '0';
    }
    while ( !TXSTAbits.TRMT );
    if ( (valeur & 0b00010000) > 0 ){
        TXREG = '1';
    }
    else{
        TXREG = '0';
    }
    while ( !TXSTAbits.TRMT );
    if ( (valeur & 0b00001000) > 0 ){
        TXREG = '1';
    }
    else{
        TXREG = '0';
    }
    while ( !TXSTAbits.TRMT );
    if ( (valeur & 0b00000100) > 0 ){
        TXREG = '1';
    }
    else{
        TXREG = '0';
    }
    while ( !TXSTAbits.TRMT );
    if ( (valeur & 0b00000010) > 0 ){
        TXREG = '1';
    }
    else{
        TXREG = '0';
    }
    while ( !TXSTAbits.TRMT );
    if ( (valeur & 0b00000001) > 0 ){
        TXREG = '1';
    }
    else{
        TXREG = '0';
    }
    while ( !TXSTAbits.TRMT );
    TXREG = 10;
    while ( !TXSTAbits.TRMT );
    TXREG = 13;
    while ( !TXSTAbits.TRMT );
}
